# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

__all__ = ["BillingCycleRelativeDate"]

BillingCycleRelativeDate = Literal["start_of_term", "end_of_term"]
